@@ -3,7 +3,7 @@
 """
 Copyright (C) 2009-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-SPDX-License-Identifier: GPL-2.0
+SPDX-License-Identifier: GPL-2.0-only
 
 """
 
@@ -21,7 +21,7 @@ from qt import QDialog, QVBoxLayout, QGridLayout, \
 from kde import KIcon, KDialog
 from dialogs import Sorry, Information, QuestionYesNo, KDialogIgnoringEscape
 from guiutil import decorateWindow
-from log import i18n, logWarning, logException, logDebug, logError
+from log import i18n, logWarning, logException, logDebug, logError, logFailure
 from message import Message, ChatMessage
 from chat import ChatWindow
 from common import Options, SingleshotOptions, Internal, Debug, isAlive
@@ -35,6 +35,7 @@ from rule import Ruleset
 from player import PlayingPlayer
 from game import PlayingGame
 from visible import VisiblePlayingGame
+from tile import Tile
 
 
 
@@ -42,7 +43,7 @@ if TYPE_CHECKING:
     from qt import QEvent, QKeyEvent
     from deferredutil import Request
     from move import Move
-    from tile import Tile, Meld, MeldList
+    from tile import Meld, MeldList
     from uitile import UITile
     from message import ClientMessage, ServerMessage
     from scene import PlayingScene
@@ -54,7 +55,7 @@ class SelectChow(KDialogIgnoringEscape):
     """asks which of the possible chows is wanted"""
 
     def __init__(self, chows:'MeldList', propose:Optional['Meld'], deferred:Deferred) ->None:
-        KDialogIgnoringEscape.__init__(self)
+        super().__init__()
         decorateWindow(self)
         self.setWindowFlags(cast(Qt.WindowType, Qt.WindowType.CustomizeWindowHint | Qt.WindowType.WindowTitleHint))
         self.setButtons(KDialog.NoButton)
@@ -93,7 +94,7 @@ class SelectKong(KDialogIgnoringEscape):
     """asks which of the possible kongs is wanted"""
 
     def __init__(self, kongs:'MeldList', deferred:Deferred) ->None:
-        KDialogIgnoringEscape.__init__(self)
+        super().__init__()
         decorateWindow(self)
         self.setButtons(KDialog.NoButton)
         self.kongs = kongs
@@ -127,7 +128,7 @@ class DlgButton(QPushButton):
     """special button for ClientDialog"""
 
     def __init__(self, message:'ClientMessage', parent:'ClientDialog') ->None:
-        QPushButton.__init__(self, parent)
+        super().__init__(parent)
         self.message = message
         self.client:'HumanClient' = parent.client
         self.setMinimumHeight(25)
@@ -141,17 +142,18 @@ class DlgButton(QPushButton):
         self.setToolTip(txt)
         self.setWarning(warn)
 
-    def keyPressEvent(self, event:'QKeyEvent') ->None:
+    def keyPressEvent(self, event:Optional['QKeyEvent']) ->None:
         """forward horizintal arrows to the hand board"""
-        key = Board.mapChar2Arrow(event)
-        if key in [Qt.Key.Key_Left, Qt.Key.Key_Right]:
-            game = self.client.game
-            if game and game.activePlayer == game.myself:
-                if game.myself.handBoard:
-                    game.myself.handBoard.keyPressEvent(event)
-                self.setFocus()
-                return
-        QPushButton.keyPressEvent(self, event)
+        if event:
+            key = Board.mapChar2Arrow(event)
+            if key in [Qt.Key.Key_Left, Qt.Key.Key_Right]:
+                game = self.client.game
+                if game and game.activePlayer == game.myself:
+                    if game.myself.handBoard:
+                        game.myself.handBoard.keyPressEvent(event)
+                    self.setFocus()
+                    return
+            super().keyPressEvent(event)
 
     def setWarning(self, warn:bool) ->None:
         """if warn, show a warning icon on the button"""
@@ -166,7 +168,7 @@ class ClientDialog(QDialog):  # pylint:disable=too-many-instance-attributes
     """a simple popup dialog for asking the player what he wants to do"""
 
     def __init__(self, client:'HumanClient', parent:Optional[QWidget]=None) ->None:
-        QDialog.__init__(self, parent)
+        super().__init__(parent)
         decorateWindow(self, i18n('Choose'))
         self.tables:List[ClientTable]
         self.setObjectName('ClientDialog')
@@ -187,9 +189,9 @@ class ClientDialog(QDialog):  # pylint:disable=too-many-instance-attributes
         self.move:'Move'  # type:ignore[assignment]
         self.sorry:Optional[Sorry] = None
 
-    def keyPressEvent(self, event:'QKeyEvent') ->None:
+    def keyPressEvent(self, event:Optional['QKeyEvent']) ->None:
         """ESC selects default answer"""
-        if not self.client.game or self.client.game.autoPlay:
+        if not event or not self.client.game or self.client.game.autoPlay:
             return
         if event.key() in [Qt.Key.Key_Escape, Qt.Key.Key_Space]:
             self.selectButton()
@@ -200,7 +202,7 @@ class ClientDialog(QDialog):  # pylint:disable=too-many-instance-attributes
                     self.selectButton(btn.message)
                     event.accept()
                     return
-            QDialog.keyPressEvent(self, event)
+            super().keyPressEvent(event)
 
     def __declareButton(self, message:'ClientMessage') ->None:
         """define a button"""
@@ -263,9 +265,9 @@ class ClientDialog(QDialog):  # pylint:disable=too-many-instance-attributes
             result = [x for x in self.buttons if x.message == answer][0]
             result.setFocus()
             if answer in [Message.Discard, Message.OriginalCall]:
-                for uiTile in game.myself.handBoard.uiTiles:
-                    if uiTile.tile is parameter:
-                        game.myself.handBoard.focusTile = uiTile
+                handBoard = game.myself.handBoard
+                assert isinstance(parameter, Tile)
+                handBoard.focusTile = handBoard[parameter][0]  # type:ignore[index]
         return result
 
     def askHuman(self, move:'Move', answers:List['ClientMessage'], deferred:Deferred) ->None:
@@ -304,6 +306,7 @@ class ClientDialog(QDialog):  # pylint:disable=too-many-instance-attributes
             return
         mainWindow = Internal.scene.mainWindow
         cwi = mainWindow.centralWidget()
+        assert cwi
         view = mainWindow.centralView
         geometry = self.geometry()
         if not self.btnHeight:
@@ -350,7 +353,7 @@ class ClientDialog(QDialog):  # pylint:disable=too-many-instance-attributes
         geometry.setHeight(int(height))
         self.setGeometry(geometry)
 
-    def showEvent(self, unusedEvent:'QEvent') ->None:
+    def showEvent(self, unusedEvent:Optional['QEvent']) ->None:
         """try to place the dialog such that it does not cover interesting information"""
         self.placeInField()
 
@@ -402,7 +405,7 @@ class HumanClient(Client):
     humanClients : List['HumanClient'] = []
 
     def __init__(self) ->None:
-        Client.__init__(self)
+        super().__init__()
         HumanClient.humanClients.append(self)
         self.table = None
         self.ruleset:Ruleset
@@ -441,7 +444,7 @@ class HumanClient(Client):
                     client.logout(
                     ).addCallback(
                         disconnectedClient,
-                        client).addErrback(logException))
+                        client).addErrback(logFailure))
         return DeferredList(deferreds)
 
     def __loggedIn(self, connection:Connection) ->None:
@@ -536,7 +539,7 @@ class HumanClient(Client):
 
     def remote_tableRemoved(self, tableid:int, message:str, *args:Any) ->None:
         """update table list"""
-        Client.remote_tableRemoved(self, tableid, message, *args)
+        super().remote_tableRemoved(tableid, message, *args)
         self.__updateTableList()
         if message:
             if self.name not in args or not message.endswith('has logged out'):
@@ -544,7 +547,7 @@ class HumanClient(Client):
 
     def __receiveTables(self, tables:List[List[Any]]) ->None:
         """now we already know all rulesets for those tables"""
-        Client.remote_newTables(self, tables)
+        super().remote_newTables(tables)
         if not Internal.autoPlay:
             if self.hasLocalServer():
                 # when playing a local game, only show pending tables with
@@ -565,11 +568,11 @@ class HumanClient(Client):
         rulesetHashes = {x[1] for x in tables}
         needRulesets = [x for x in rulesetHashes if not Ruleset.hashIsKnown(x)]
         if needRulesets:
-            self.callServer(
-                'needRulesets',
-                needRulesets).addCallback(
-                    gotRulesets).addErrback(logException).addCallback(
-                        self.__receiveTables).addErrback(logException)
+            _ = self.callServer('needRulesets', needRulesets)
+            _ = _.addCallback(gotRulesets)
+            _ = _.addErrback(logFailure)
+            _ = _.addCallback(self.__receiveTables)
+            _.addErrback(logFailure)
         else:
             self.__receiveTables(tables)
 
@@ -582,7 +585,7 @@ class HumanClient(Client):
 
     def tableChanged(self, table:ClientTable) ->Tuple[Optional[ClientTable], ClientTable]:
         """update table list"""
-        oldTable, newTable = Client.tableChanged(self, table)
+        oldTable, newTable = super().tableChanged(table)
         if oldTable and oldTable == self.table:
             # this happens if a table has more than one human player and
             # one of them leaves the table. In that case, the other players
@@ -603,7 +606,7 @@ class HumanClient(Client):
                         if self.beginQuestion:
                             self.beginQuestion.cancel()
                         Sorry(i18n('Player %1 has left the table', name)).addCallback(
-                            sorried).addErrback(logException).addCallback(self.showTableList).addErrback(logException)
+                            sorried).addErrback(logFailure).addCallback(self.showTableList).addErrback(logFailure)
                         break
         self.__updateTableList()
         return oldTable, newTable
@@ -643,7 +646,7 @@ class HumanClient(Client):
                 return clientReady()
             return Message.NoGameStart
 
-        def cancelled(unused:'Request') ->'ServerMessage':
+        def cancelled(unused:Failure) ->'ServerMessage':
             """the user does not want to start now. Back to table list"""
             if Debug.table:
                 logDebug(f'{self.name}: Readyforgamestart returns Message.NoGameStart '
@@ -671,11 +674,11 @@ class HumanClient(Client):
             answered).addErrback(cancelled)
         return self.beginQuestion
 
-    def readyForHandStart(self, playerNames:List[Tuple['Wind', str]], rotateWinds:bool) ->Optional[Deferred]:
+    def readyForHandStart(self, playerNames:List[Tuple['Wind', str]], mustRotateWinds:bool) ->Optional[Deferred]:
         """playerNames are in wind order ESWN. Never called for first hand."""
         def answered(unused:Optional[List['Request']]=None) ->Optional[Deferred]:
             """called after the client player said yes, I am ready"""
-            return Client.readyForHandStart(self, playerNames, rotateWinds) if self.connection else None
+            return Client.readyForHandStart(self, playerNames, mustRotateWinds) if self.connection else None
         if not self.connection:
             # disconnected meanwhile
             return None
@@ -685,13 +688,13 @@ class HumanClient(Client):
                 Internal.mainWindow.updateGUI()
         assert self.game
         assert not self.game.isFirstHand()
-        return Information(i18n("Ready for next hand?"), modal=False).addCallback(answered).addErrback(logException)
+        return Information(i18n("Ready for next hand?"), modal=False).addCallback(answered).addErrback(logFailure)
 
     def ask(self, move:'Move', answers:List['ClientMessage']) ->Deferred:
         """server sends move. We ask the user. answers is a list with possible answers,
         the default answer being the first in the list."""
         if not Options.gui:
-            return Client.ask(self, move, answers)
+            return super().ask(move, answers)
         scene = Internal.scene
         assert scene
         assert self.game
@@ -782,10 +785,10 @@ class HumanClient(Client):
         return answer, args
 
 
-    def __answerError(self, answer:Message, move:'Move', answers:List[Message]) ->None:
+    def __answerError(self, result: Any, move:'Move', answers:List['ClientMessage']) ->None:
         """an error happened while determining the answer to server"""
         logException(
-            f"{self.game.myself.name if self.game else 'NOGAME'} {answer} {move} {answers}")
+            f"{result} {self.game.myself.name if self.game else 'NOGAME'} {move} {answers}")
 
     def remote_abort(self, tableid:int, message: str, *args:Any) ->None:
         """the server aborted this game"""
@@ -804,14 +807,16 @@ class HumanClient(Client):
             """now that the user clicked the 'game over' prompt away, clean up"""
             if self.game:
                 self.game.rotateWinds()
+                if self.game.finished():
+                    self.game.finish_in_db()
                 _ = self.game.close()
                 if Internal.mainWindow:
-                    _.addCallback(Internal.mainWindow.close).addErrback(logException)
+                    _.addCallback(Internal.mainWindow.close).addErrback(logFailure)
         assert self.table and self.table.tableid == tableid
         if Internal.scene:
             # update the balances in the status bar:
             Internal.scene.mainWindow.updateGUI()
-        Information(i18n(message, *args)).addCallback(yes).addErrback(logException)
+        Information(i18n(message, *args)).addCallback(yes).addErrback(logFailure)
 
     def remote_serverDisconnects(self, result:Optional[str]=None) ->None:
         """we logged out or lost connection to the server.
@@ -898,7 +903,7 @@ class HumanClient(Client):
             ruleset = selectDialog.cbRuleset.current
         deferred = self.__requestNewTableFromServer(ruleset=ruleset)
         if self.hasLocalServer():
-            deferred.addCallback(self.__newLocalTable).addErrback(logException)
+            deferred.addCallback(self.__newLocalTable).addErrback(logFailure)
         assert self.tableList
         self.tableList.requestedNewTable = True
 
@@ -923,7 +928,7 @@ class HumanClient(Client):
             self.connection = None
             if Debug.connections:
                 logDebug(f'sending logout to server for {self}')
-            return self.callServer('logout').addCallback(loggedout, conn).addErrback(logException)
+            return self.callServer('logout').addCallback(loggedout, conn).addErrback(logFailure)
         return succeed(None)
 
     def __logCallServer(self, *args:Any) ->None:
