@@ -3,11 +3,11 @@
 """
 Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-SPDX-License-Identifier: GPL-2.0
+SPDX-License-Identifier: GPL-2.0-only
 
 """
 
-from typing import Dict, TYPE_CHECKING, Any
+from typing import Dict, TYPE_CHECKING, Any, Optional
 from qt import QObject, QByteArray, QEvent, QSplitter, QHeaderView
 
 from common import Internal, isAlive
@@ -24,7 +24,7 @@ class StateSaver(QObject):
     savers : Dict['QWidget', 'StateSaver'] = {}
 
     def __init__(self, *widgets:'QWidget') ->None:
-        QObject.__init__(self)
+        super().__init__()
         pref = Internal.Preferences
         assert pref
         self.widgets = []
@@ -67,25 +67,26 @@ class StateSaver(QObject):
     @staticmethod
     def __generateName(widget:'QWidget') ->str:
         """generate a name for this widget to be used in the config file"""
-        orgWidget = widget
+        _ = widget
         name = english(widget.objectName())
         if not name:
-            while widget.parentWidget():
-                name = widget.__class__.__name__ + name
-                widget = widget.parentWidget()
-                if widget.parentWidget():
-                    widgetName = english(widget.parentWidget().objectName())
+            while _.parentWidget():
+                name = _.__class__.__name__ + name
+                assert _  # for mypy - not possible because of while condition
+                _ = _.parentWidget()
+                if _.parentWidget():
+                    widgetName = english(_.parentWidget().objectName())
                     if widgetName:
                         name = widgetName + name
                         break
         if not name:
-            name = orgWidget.__class__.__name__
+            name = widget.__class__.__name__
         return str(name)
 
-    def eventFilter(self, unusedWatched:QObject, event:QEvent) ->bool:
+    def eventFilter(self, unusedWatched:Optional[QObject], event:Optional[QEvent]) ->bool:
         """if the watched widget hides, save its state.
         Return False if the event should be handled further"""
-        if QEvent is not None:
+        if QEvent is not None and event:
             # while appquit, QEvent may be None. Maybe not anymore
             # with later versions?
             if event.type() in(QEvent.Type.Hide, QEvent.Type.Move, QEvent.Type.Resize):
